@@ -99,7 +99,7 @@ class Chop(object):
             prec_map = {
                 'q43': (4, 7), 'fp8-e4m3': (4, 7), 'q52': (3, 15), 'fp8-e5m2': (3, 15),
                 'h': (11, 15), 'half': (11, 15), 'fp16': (11, 15),
-                'b': (8, 127), 'bfloat16': (8, 127),
+                'b': (8, 127), 'bfloat16': (8, 127), 'bf16': (8, 127),
                 's': (24, 127), 'single': (24, 127), 'fp32': (24, 127),
                 'd': (53, 1023), 'double': (53, 1023), 'fp64': (53, 1023)
             }
